Enum eventsourced::SpawnError
source · pub enum SpawnError {
LoadSnapshot(Box<dyn StdError + Send + Sync>),
InvalidLastSeqNo(Option<NonZeroU64>, Option<NonZeroU64>),
LastNonZeroU64(Box<dyn StdError + Send + Sync>),
EvtsById(Box<dyn StdError + Send + Sync>),
NextEvt(Box<dyn StdError + Send + Sync>),
}Expand description
Error from spawning an event sourced entity.
Variants
LoadSnapshot(Box<dyn StdError + Send + Sync>)
A snapshot cannot be loaded from the snapshot store.
InvalidLastSeqNo(Option<NonZeroU64>, Option<NonZeroU64>)
The last sequence number is less than the snapshot sequence number.
LastNonZeroU64(Box<dyn StdError + Send + Sync>)
The last seqence number cannot be obtained from the event log.
EvtsById(Box<dyn StdError + Send + Sync>)
Events by ID cannot be obtained from the event log.
NextEvt(Box<dyn StdError + Send + Sync>)
The next event cannot be obtained from the event log.